# Copyright (c) IFM Lab. All rights reserved.

from .gstvqa_metric import GstVqa
# from .gstvqa_crossdata_metric import GSTVQACrossData
# from .GSTVQA.TCSVT_Release.GVQA_Release.GVQA_Cross.cross_test import Test_VQADataset

__all__ = ['GstVqa']