# Copyright (c) IFM Lab. All rights reserved.

# from .modular_bvqa_metric import ModularBVQA
# rom .GSTVQA.TCSVT_Release.GVQA_Release.GVQA_Cross.cross_test import Test_VQADataset

# __all__ = ['ModularBVQA']